Pokedex = LibStub("AceAddon-3.0"):NewAddon("Pokedex", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Pokedex")

--=========================================================================--
-- Keybinding globals --
--=========================================================================--
BINDING_HEADER_POKEDEX = L["Pokedex"]
BINDING_NAME_POKEDEXSUPERTOGGLE = L["Super Toggle"]
BINDING_NAME_POKEDEXSUMMONMOUNT = L["Summon Mount"]
BINDING_NAME_POKEDEXDISMISSMOUNT = L["Dismiss Mount"]
BINDING_NAME_POKEDEXTOGGLEMOUNT = L["Toggle Mount"]
BINDING_NAME_POKEDEXSUMMONNEXTMOUNT = L["Summon Next Mount"]
BINDING_NAME_POKEDEXSUMMONOTHERMOUNT = L["Summon Other Mount"]
BINDING_NAME_POKEDEXSUMMONCOMPANION = L["Summon Companion"]
BINDING_NAME_POKEDEXDISMISSCOMPANION = L["Dismiss Companion"]
BINDING_NAME_POKEDEXTOGGLECOMPANION = L["Toggle Companion"]
BINDING_NAME_POKEDEXSUMMONNEXTCOMPANION = L["Summon Next Companion"]
BINDING_NAME_POKEDEXSUMMONVENDOR = L["Summon Vendor"]
BINDING_NAME_POKEDEXCHANGETITLE = L["Change Title"]

--=========================================================================--
-- Fake Enum Types		TODO: find a way to make these constants
--=========================================================================--

-- Mount Speeds
local MS_Unknown      =  0  -- mounts we failed to identify
local MS_Flyer310     =  1	-- extremely fast flyers like the Ashes of Al'ar
local MS_Flyer280     =  2	-- very fast flyers
local MS_Flyer150     =  3	-- regular flyers
local MS_Runner100    =  4	-- very fast running mounts
local MS_Runner60     =  5	-- regular runners
local MS_Runner0      =  6	-- running mounts that don't actually increase speed like the Riding Turtle
local MS_RunnerQiraji =  7 	-- Qiraji scarab mounts
local MS_MAX          =  7 	-- used for range checking

-- Initialization States
local IS_INITIALIZED   = 0;
local IS_INITIALIZING  = 1;
local IS_UNINITIALIZED = 2;

-- debug categories
local DC_MISC     = 0;
local DC_MOUNTS   = 1;
local DC_PETS     = 2;
local DC_TITLES   = 3;
local DC_DISMOUNT = 4;

-- debug levels
local DL_NONE   = 0		-- no debug output
local DL_BASIC  = 1		-- most basic level of information
local DL_EXCEP  = 2		-- interesting exceptions and special cases
local DL_AV     = 3		-- annoyingly verbose
local DL_MAX	= DL_AV	-- duplicate definition, used for range checking

--=========================================================================--
-- "constants"
--=========================================================================--
local gc_rgstrChannelDescs = { L["personal"], L["party"], L["raid"], L["emote"], L["say"], L["yell"] }
local gc_rgstrChannels = { "ERROR", "PARTY", "RAID", "EMOTE", "SAY", "YELL" } -- CHATID types, not localized, ERROR is just placeholder

local gc_rgstrMountSpeedDescs = { [0] = L["Unidentified Mounts"], L["Extremely Fast Flyers"], L["Very Fast Flyers"], L["Flyers"], L["Very Fast Runners"], L["Runners"], L["Slow Runners"], L["Qiraji Scarabs"] }
local gc_rgstrMountSpeeds = { [0] = "MS_Unknown", "MS_Flyer310", "MS_Flyer280", "MS_Flyer150", "MS_Runner100", "MS_Runner60", "MS_Runner0", "MS_RunnerQiraji" } -- not localizing these strings since they are for debug use only

local gc_itemIDSnowball = 17202
local gc_iDefaultRank = 5


--=========================================================================--
-- globals
--=========================================================================--

-- addon infrastructure
local g_iCurDataFormat = 4
local g_InitState = IS_UNINITIALIZED;
local DL = {}; -- keys are DC (debug categories), values are DL (debug level)

-- skills
local g_iRidingSkill = 0
local g_iMiningSkill = 0
local g_iHerbalismSkill = 0
local g_iSkinningSkill = 0

-- mounts
local g_rgstrMountTypes		-- indexed by mount speed, not contiguously from 0 or 1
local g_iMountType			-- current index showing in dropdown, will be 0 or an MS_ value
local g_rgMountsByType		-- g_rgMountsByType[MS_speed][rgIndices | rgNames][array]
local g_iMountOfType		-- index of mount in its speed group
local g_rgstrHotMountNames
local g_iHotMount
local g_rgMountMap			-- keyed by mount index, values are mount speed and index under g_rgMountsByType
local g_iTravelersMammoth = 0

-- companions
local g_rgstrCompanionNames
local g_iCompanionForRanking
local g_rgstrHotCompanionNames
local g_iHotCompanion
local g_iArgentSquire = 0;

-- titles
local g_rgstrTitleNames
local g_iTitleForRanking
local g_rgstrHotTitleNames
local g_iHotTitle
local g_rgTitleMap	-- keyed by id, value is index of corresponding name in sorted order

-- dismount
local g_iAutoDismountFlying
local g_fCanDismountForCombat = false	-- means dismount is currently turned on because the in combat condition is true
local g_fCanDismountForAttack = false	-- means dismount is currently turned on because the can attack condition is true

-- profile saved settings
local defaults = {
	profile = {
		rgDebugInfo = { MISC     = { category = DC_MISC,     level = DL_NONE },
		                MOUNTS   = { category = DC_MOUNTS,   level = DL_NONE },
		                PETS     = { category = DC_PETS,     level = DL_NONE },
		                TITLES   = { category = DC_TITLES,   level = DL_NONE },
		                DISMOUNT = { category = DC_DISMOUNT, level = DL_NONE } },
		iDataFormat = 0,
		fFirstBoot = true,
		fManageAutoDismount = false,
		fDismountForGathering = false,
		fDismountForCombat = false,
		fDismountForAttack = false,
		fAnnounce = true,
		iChannel = 1,
		fChangeTitleOnMount = false,
		fEnableHotness = true,
		fNewHotness = true,
		idHotMount = 0,
		iHotMountPref = 50,
		idHotCompanion = 0,
		iHotCompanionPref = 50,
		idHotTitle = 0,
		iHotTitlePref = 50,
		rgMountInfo = {},			-- { name, id, rank }
		rgCompanionInfo = {},		-- { name, id, rank, snowball }
		rgTitleInfo = {}			-- { name, id, rank }  -- index matches the index of the trimmed name after sorting

	},
}


-- As the addon develops and ages, there may be changes to the structure 
-- and contents of the saved data. This function migrates data between
-- data format versions to preserve the users settings.
function Pokedex:MigrateSavedData()
	-- if already up to date, exit
	if (self.db.profile.iDataFormat == g_iCurDataFormat) then
		return;
	end

	-- if its first boot, theres no data to migrate
	if not self.db.profile.fFirstBoot then

		-- rename of critter to companion
		if self.db.profile.iDataFormat < 1 then
			if (DL[DC_MISC] >= DL_BASIC) then self:Print("migrating to data format 1"); end

			if self.db.profile.idHotCritter ~= nil then
				self.db.profile.idHotCompanion = self.db.profile.idHotCritter;
				self.db.profile.idHotCritter = nil;
			end
			if self.db.profile.idHotCritterPref ~= nil then
				self.db.profile.iHotCompanionPref = self.db.profile.idHotCritterPref;
				self.db.profile.idHotCritterPref = nil;
			end
			if self.db.profile.rgCritterInfo ~= nil then
				self.db.profile.rgCompanionInfo = self.db.profile.rgCritterInfo;
				self.db.profile.rgCritterInfo = nil;
			end
		end

		-- change of how we characterize mount performance
		if self.db.profile.iDataFormat < 2 then
			if (DL[DC_MISC] >= DL_BASIC) then self:Print("migrating to data format 2"); end
			for index = 1, #self.db.profile.rgMountInfo, 1 do
				self.db.profile.rgMountInfo[index][4] = nil;
			end
		end

		-- we'll no longer try to change title on load, instead when mounting
		-- change to the way we save mount, companion and title info
		if self.db.profile.iDataFormat < 3 then
			if (DL[DC_MISC] >= DL_BASIC) then self:Print("migrating to data format 3"); end

			self.db.profile.fChangeTitleOnMount = self.db.profile.fChangeTitleOnLoad;
			self.db.profile.fChangeTitleOnLoad = nil;

			local rgNew;
			
			-- migrate mount info
			rgNew = {};
			for i = 1, #self.db.profile.rgMountInfo, 1 do
				rgNew[i] = { name = self.db.profile.rgMountInfo[i][1], 
				               id = self.db.profile.rgMountInfo[i][2],
				             rank = self.db.profile.rgMountInfo[i][3] };
			end
			self.db.profile.rgMountInfo = rgNew;
			
			-- migrate companion info
			rgNew = {};
			for i = 1, #self.db.profile.rgCompanionInfo, 1 do
				rgNew[i] = { name = self.db.profile.rgCompanionInfo[i][1],
				               id = self.db.profile.rgCompanionInfo[i][2],
				             rank = self.db.profile.rgCompanionInfo[i][3],
				         snowball = self.db.profile.rgCompanionInfo[i][4] };
			end
			self.db.profile.rgCompanionInfo = rgNew;
			
			-- migrate title info
			rgNew = {};
			for i = 1, #self.db.profile.rgTitleInfo, 1 do
				rgNew[i] = { name = self.db.profile.rgTitleInfo[i][1],
				               id = self.db.profile.rgTitleInfo[i][2],
				             rank = self.db.profile.rgTitleInfo[i][3] };
			end
			self.db.profile.rgTitleInfo = rgNew;
		end
		
		-- users should have right to choose to opt in to safe dismount code
		if self.db.profile.iDataFormat < 4 then
			if (DL[DC_MISC] >= DL_BASIC) then self:Print("migrating to data format 4"); end

			self.db.profile.fManageAutoDismount = false;
			self.db.profile.fDismountForGathering = false;
			self.db.profile.fDismountForCombat = false;
			self.db.profile.fDismountForAttack = false;
		end
		
	end

	self.db.profile.iDataFormat = g_iCurDataFormat;
end


--=========================================================================--
-- Options UI definition
--=========================================================================--

local function GetOptions()
	if (g_InitState ~= IS_INITIALIZED and Pokedex:Initialize("GetOptions") == IS_UNINITIALIZED) then
		return {
			name = L["Pokedex  v3.3.04"],
			handler = Pokedex,
			type = 'group',
			childGroups = "tab",
			args = {
				CtrlInitError = {
					order = 1,
					type = "description",
					name = L["ERROR: Pokedex failed to initialize correctly. This is usually caused when WoW has invalidated its cache and hasn't finished rebuilding it. Please try this action again later."],
				},
				Padding_InitError = {
					order = 2,
					type = "description",
					name = "",
				},
				CtrlRetryInit = {
					order = 3,
					type = "execute",
					name = L["retry initialization"],
					desc = L["retry initialization"],
					func = function() InterfaceOptionsFrame_OpenToCategory(Pokedex.optionsFrame); end,
				}
			}
		}
	end

	if (not Pokedex.options) then
		Pokedex.options = {
			name = L["Pokedex  v3.3.04"],
			handler = Pokedex,
			type = 'group',
			childGroups = "tab",
			args = {
				test = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["echoes out test info relevant to current feature in development"],
					desc = L["echoes out test info relevant to current feature in development"],
					func = function(info) Pokedex:EchoTest() end,
				},
				debug = {
					hidden = true,
					order = 0,
					type = "input",
					name = L["echoes out test info relevant to current feature in development"],
					desc = L["echoes out test info relevant to current feature in development"],
					get = function(info) return "" end,
					set = "SetDebug",
				},
				speed = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["echoes out current speed"],
					desc = L["echoes out current speed"],
					func = function(info) Pokedex:EchoSpeed() end,
				},
				zone = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["echoes out zone info"],
					desc = L["echoes out zone info"],
					func = function(info) Pokedex:EchoZone() end,
				},
				SuperToggle = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["toggle mount or companion"],
					desc = L["toggles mount or companion"],
					func = function(info) Pokedex:SuperToggle() end,
				},
				SummonMount = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon mount"],
					desc = L["summons a mount"],
					func = function(info) Pokedex:SummonMount(false, false) end,
				},
				sm = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon mount"],
					desc = L["summons a mount"],
					func = function(info) Pokedex:SummonMount(false, false) end,
				},
				DismissMount = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["dismiss mount"],
					desc = L["dismisses current mount"],
					func = function(info) Pokedex:DismissMount() end,
				},
				dm = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["dismiss mount"],
					desc = L["dismisses current mount"],
					func = function(info) Pokedex:DismissMount() end,
				},
				ToggleMount = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["toggle mount"],
					desc = L["toggles a mount"],
					func = function(info) Pokedex:ToggleMount() end,
				},
				tm = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["toggle mount"],
					desc = L["toggles a mount"],
					func = function(info) Pokedex:ToggleMount() end,
				},
				SummonNextMount = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon next mount"],
					desc = L["summons next mount in collection"],
					func = function(info) Pokedex:SummonMount(true, false) end,
				},
				snm = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon next mount"],
					desc = L["summons next mount in collection"],
					func = function(info) Pokedex:SummonMount(true, false) end,
				},
				SummonOtherMount = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon other mount"],
					desc = L["if summon mount would summon flyer, will summon ground mount and vice versa"],
					func = function(info) Pokedex:SummonMount(false, true) end,
				},
				som = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon other mount"],
					desc = L["if summon mount would summon flyer, will summon ground mount and vice versa"],
					func = function(info) Pokedex:SummonMount(false, true) end,
				},
				SummonCompanion = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon companion"],
					desc = L["summons a companion"],
					func = function(info) Pokedex:SummonCompanion(false) end,
				},
				sc = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon companion"],
					desc = L["summons a companion"],
					func = function(info) Pokedex:SummonCompanion(false) end,
				},
				DismissCompanion = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["dismiss companion"],
					desc = L["dismisses current companion"],
					func = function(info) DismissCompanion("CRITTER") end,
				},
				dc = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["dismiss companion"],
					desc = L["dismisses current companion"],
					func = function(info) DismissCompanion("CRITTER") end,
				},
				ToggleCompanion = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["toggle companion"],
					desc = L["toggles a companion"],
					func = function(info) Pokedex:ToggleCompanion() end,
				},
				tc = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["toggle companion"],
					desc = L["toggles a companion"],
					func = function(info) Pokedex:ToggleCompanion() end,
				},
				SummonNextCompanion = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon next companion"],
					desc = L["summons next companion in collection"],
					func = function(info) Pokedex:SummonCompanion(true) end,
				},
				snc = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon next companion"],
					desc = L["summons next companion in collection"],
					func = function(info) Pokedex:SummonCompanion(true) end,
				},
				vendor = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["summon vendor"],
					desc = L["summons a mount or companion with vendor capabilities"],
					func = function(info) Pokedex:SummonVendor() end,
				},
				ChangeTitle = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["change title"],
					desc = L["change title"],
					func = function() Pokedex:ChangeTitle() end,
				},
				ct = {
					hidden = true,
					order = 0,
					type = "execute",
					name = L["change title"],
					desc = L["change title"],
					func = function() Pokedex:ChangeTitle() end,
				},

				CtrlReadMe = {
					order = 5,
					type = "description",
					name = L["Please see the README.TXT file in the Pokedex addon folder for more information on how to use Pokedex"],
				},
				
				CtrlAnnounce = {
					order = 10,
					type = "toggle",
					name = L["announce"],
					desc = L["(on|off) let everyone know who *you* choose"],
					get = function(info) return Pokedex.db.profile.fAnnounce end,
					set = function(info, value) Pokedex.db.profile.fAnnounce = value end,
				},
				CtrlChannel = {
					order = 11,
					type = "select",
					name = L["channel"],
					desc = L["channel to announce selection in"],
					values = gc_rgstrChannelDescs,
					get = function(info) return Pokedex.db.profile.iChannel end,
					set = function(info, value) Pokedex.db.profile.iChannel = value end,
				},
				Padding_Announcements = {
					order = 12,
					type = "description",
					name = "",
				},
				CtrlEnableHotness = {
					order = 15,
					type = "toggle",
					name = L["enable hot pets"],
					desc = L["lets your turn on|off the hot pet subfeatures"],
					get = function(info) return Pokedex.db.profile.fEnableHotness end,
					set = function(info, value) Pokedex.db.profile.fEnableHotness = value end,
				},
				CtrlNewHotness = {
					order = 16,
					type = "toggle",
					name = L["new hotness"],
					desc = L["always make newest mount or companion the hot one"],
					disabled = function(info) return not Pokedex.db.profile.fEnableHotness end,
					get = function(info) return Pokedex.db.profile.fNewHotness end,
					set = function(info, value) Pokedex.db.profile.fNewHotness = value end,
				},
				Padding_Hotness = {
					order = 17,
					type = "description",
					name = "",
				},

				CtrlGroupMounts = {
					order = 20,
					name = L["Mounts"],
					handler = Pokedex,
					type = 'group',
					args = {
						CtrlSummonMount = {
							order = 1,
							type = "execute",
							name = L["summon mount"],
							desc = L["summons a mount"],
							func = function(info) Pokedex:SummonMount(false, false) end,
						},
						CtrlDismissMount = {
							order = 2,
							type = "execute",
							name = L["dismiss mount"],
							desc = L["dismisses current mount"],
							func = function(info) Pokedex:DismissMount() end,
						},
						Padding_MountButtons = {
							order = 3,
							type = "description",
							name = "",
						},
						CtrlMountTypes = {
							order = 5,
							type = "select",
							name = L["select mount type"],
							desc = L["select mount type"],
							values = function(info) return g_rgstrMountTypes end,
							disabled = function(info) return (g_iMountType == 0) end,
							get = function(info) return g_iMountType end,
							set = function(info, value) 
								g_iMountType = value 
								g_iMountOfType = 1
							end,
						},
						Padding_MountTypes = {
							order = 6,
							type = "description",
							name = "",
						},
						CtrlMountsOfType = {
							order = 10,
							type = "select",
							name = L["select mount for ranking"],
							desc = L["mount whose rank you can set"],
							values = function(info) return g_rgMountsByType[g_iMountType].rgNames end,
							disabled = function(info) return (g_iMountOfType == 0) end,
							get = function(info) return g_iMountOfType end,
							set = function(info, value) g_iMountOfType = value end,
						},
						CtrlRankingForMount = {
							order = 11,
							type = "range",
							name = L["mount rank"],
							desc = L["rank of current mount"],
							min = 0,
							max = 10,
							step = 1,
							disabled = function(info) return (g_iMountType == 0 or g_iMountOfType == 0) end,
							get = function(info) 
								local iMount = g_rgMountsByType[g_iMountType].rgIndices[g_iMountOfType]
								return Pokedex.db.profile.rgMountInfo[iMount].rank
							end,
							set = function(info, value) 
								local iMount = g_rgMountsByType[g_iMountType].rgIndices[g_iMountOfType]
								Pokedex.db.profile.rgMountInfo[iMount].rank = value
							end,
						},
						Padding_MountRankings = {
							order = 12,
							type = "description",
							name = "",
						},
						CtrlHotMount = {
							order = 15,
							type = "select",
							name = L["select hot mount"],
							desc = L["mount to become hot one"],
							values = function(info) return g_rgstrHotMountNames end,
							disabled = function(info) return (not Pokedex.db.profile.fEnableHotness or #g_rgstrHotMountNames == 0) end,
							get = function(info) return g_iHotMount end,
							set = function(info, value) 
								g_iHotMount = value
								Pokedex.db.profile.idHotMount = Pokedex.db.profile.rgMountInfo[value].id
							end,
						},
						CtrlMountHeat = {
							order = 16,
							type = "range",
							name = L["mount heat"],
							desc = L["set hotness as a percentage - 100% means the hot pet is the only one that will be chosen"],
							min = 0,
							max = 100,
							step = 5,
							disabled = function(info) return not Pokedex.db.profile.fEnableHotness end,
							get = function(info) return Pokedex.db.profile.iHotMountPref end,
							set = function(info, value) Pokedex.db.profile.iHotMountPref = value end,
						},
						Padding_HotMounts = {
							order = 17,
							type = "description",
							name = "",
						},
					},
				},

				CtrlGroupCompanions = {
					order = 40,
					name = L["Companions"],
					handler = Pokedex,
					type = 'group',
					args = {
						CtrlSummonCompanion = {
							order = 1,
							type = "execute",
							name = L["summon companion"],
							desc = L["summons a companion"],
							func = function() Pokedex:SummonCompanion(false) end,
						},
						CtrlDismissCompanion = {
							order = 2,
							type = "execute",
							name = L["dismiss companion"],
							desc = L["dismisses current companion"],
							func = function() DismissCompanion("CRITTER") end,
						},
						Padding_CompanionButtons = {
							order = 3,
							type = "description",
							name = "",
						},
						CtrlCompanionForRanking = {
							order = 5,
							type = "select",
							name = L["select companion for ranking"],
							desc = L["companion whose rank you can set"],
							values = function(info) return g_rgstrCompanionNames end,
							disabled = function(info) return (#g_rgstrCompanionNames == 0) end,
							get = function(info) return g_iCompanionForRanking end,
							set = function(info, value) g_iCompanionForRanking = value end,
						},
						CtrlRankingForCompanion = {
							order = 6,
							type = "range",
							name = L["companion rank"],
							desc = L["rank of current companion"],
							min = 0,
							max = 10,
							step = 1,
							disabled = function(info) return (#g_rgstrCompanionNames == 0) end,
							get = function(info) return Pokedex.db.profile.rgCompanionInfo[g_iCompanionForRanking].rank end,
							set = function(info, value) Pokedex.db.profile.rgCompanionInfo[g_iCompanionForRanking].rank = value end,
						},
						Padding_CompanionRankings = {
							order = 7,
							type = "description",
							name = "",
						},
						CtrlHotCompanion = {
							order = 10,
							type = "select",
							name = L["select hot companion"],
							desc = L["companion to become hot one"],
							values = function(info) return g_rgstrHotCompanionNames end,
							disabled = function(info) return (not Pokedex.db.profile.fEnableHotness or #g_rgstrHotCompanionNames == 0) end,
							get = function(info) return g_iHotCompanion end,
							set = function(info, value)
								g_iHotCompanion = value
								Pokedex.db.profile.idHotCompanion = Pokedex.db.profile.rgCompanionInfo[value].id
							end,
						},
						CtrlCompanionHeat = {
							order = 11,
							type = "range",
							name = L["companion heat"],
							desc = L["set hotness as a percentage - 100% means the hot pet is the only one that will be chosen"],
							min = 0,
							max = 100,
							step = 5,
							disabled = function(info) return not Pokedex.db.profile.fEnableHotness end,
							get = function(info) return Pokedex.db.profile.iHotCompanionPref end,
							set = function(info, value) Pokedex.db.profile.iHotCompanionPref = value end,
						},
						Padding_HotCompanions = {
							order = 12,
							type = "description",
							name = "",
						},
					},
				},

				CtrlGroupTitles = {
					order = 60,
					name = L["Titles"],
					handler = Pokedex,
					type = 'group',
					args = {
						CtrlChangeTitleOnMount = {
							order = 1,
							type = "toggle",
							name = L["Change title on mount"],
							desc = L["Change title when a mount is summoned"],
							get = function(info) return Pokedex.db.profile.fChangeTitleOnMount end,
							set = function(info, value) Pokedex.db.profile.fChangeTitleOnMount = value end,
						},
						CtrlChangeTitle = {
							order = 2,
							type = "execute",
							name = L["change title"],
							desc = L["change title"],
							func = function() Pokedex:ChangeTitle() end,
						},
						Padding_TitleButtons = {
							order = 3,
							type = "description",
							name = "",
						},
						CtrlTitleForRanking = {
							order = 5,
							type = "select",
							name = L["select title for ranking"],
							desc = L["title whose rank you can set"],
							values = function(info) return g_rgstrTitleNames end,
							disabled = function(info) return (#g_rgstrTitleNames == 0) end,
							get = function(info) return g_iTitleForRanking end,
							set = function(info, value) g_iTitleForRanking = value end,
						},
						CtrlRankingForTitle = {
							order = 6,
							type = "range",
							name = L["title rank"],
							desc = L["rank of current title"],
							min = 0,
							max = 10,
							step = 1,
							disabled = function(info) return (#g_rgstrTitleNames == 0) end,
							get = function(info) return Pokedex.db.profile.rgTitleInfo[g_iTitleForRanking].rank end,
							set = function(info, value) Pokedex.db.profile.rgTitleInfo[g_iTitleForRanking].rank = value end,
						},
						Padding_TitleRankings = {
							order = 7,
							type = "description",
							name = "",
						},
						CtrlHotTitle = {
							order = 10,
							type = "select",
							name = L["select hot title"],
							desc = L["title to become hot one"],
							values = function(info) return g_rgstrHotTitleNames end,
							disabled = function(info) return (not Pokedex.db.profile.fEnableHotness or #g_rgstrHotTitleNames == 0) end,
							get = function(info) return g_iHotTitle end,
							set = function(info, value) 
								g_iHotTitle = value
								Pokedex.db.profile.idHotTitle = Pokedex.db.profile.rgTitleInfo[value].id
							end,
						},
						CtrlTitleHeat = {
							order = 11,
							type = "range",
							name = L["title heat"],
							desc = L["set hotness as a percentage - 100% means the hot pet is the only one that will be chosen"],
							min = 0,
							max = 100,
							step = 5,
							disabled = function(info) return not Pokedex.db.profile.fEnableHotness end,
							get = function(info) return Pokedex.db.profile.iHotTitlePref end,
							set = function(info, value) Pokedex.db.profile.iHotTitlePref = value end,
						},
						Padding_HotTitles = {
							order = 12,
							type = "description",
							name = "",
						},
					},
				},

				CtrlGroupAutoDismount = {
					order = 80,
					name = L["Safe Dismount"],
					handler = Pokedex,
					type = 'group',
					-- hidden = function(info) return (g_iRidingSkill < 225) end,
					args = {
						CtrlDismountFeatureDesc = {
							order = 1,
							type = "description",
							name = L["This feature works with and manages the games Auto Dismount in Flight setting to improve your experience with flying mounts. Auto dismount will be disabled unless in one of the selected conditions."],
						},
						CtrlAutoDismount = {
							order = 2,
							type = "toggle",
							width = "full",
							name = L["Manage Auto Dismount"],
							desc = L["Enables Pokedex's management of the Auto Dismount in Flight option"],
							get = function(info) return Pokedex.db.profile.fManageAutoDismount end,
							set = "SetManageAutoDismount",
						},
						Padding_Dismount1 = {
							order = 3,
							type = "description",
							width = "full",
							name = "",
						},
						Padding_Dismount2 = {
							order = 4,
							type = "description",
							width = "full",
							name = "",
						},
						Padding_Dismount3 = {
							order = 5,
							type = "description",
							width = "full",
							name = "",
						},
						CtrlDismountCombat = {
							order = 10,
							type = "toggle",
							width = "full",
							name = L["Dismount for Combat"],
							desc = L["Enables Auto Dismount when in combat"],
							disabled = function(info) return (not Pokedex.db.profile.fManageAutoDismount) end,
							get = function(info) return Pokedex.db.profile.fDismountForCombat end,
							set = function(info, value) Pokedex.db.profile.fDismountForCombat = value end,
						},
						CtrlDismountAttack = {
							order = 15,
							type = "toggle",
							width = "full",
							name = L["Dismount to Attack"],
							desc = L["Enables Auto Dismount when targeting something attackable"],
							disabled = function(info) return (not Pokedex.db.profile.fManageAutoDismount) end,
							get = function(info) return Pokedex.db.profile.fDismountForAttack end,
							set = function(info, value) Pokedex.db.profile.fDismountForAttack = value end,
						},
						CtrlDismountGathering = {
							order = 20,
							type = "toggle",
							width = "full",
							name = L["Dismount for Gathering"],
							desc = L["Enables Auto Dismount when gathering a resource with mining, herbalism or skinning"],
							hidden = function(info) return ((g_iMiningSkill + g_iHerbalismSkill + g_iSkinningSkill) == 0) end,
							disabled = function(info) return (not Pokedex.db.profile.fManageAutoDismount) end,
							get = function(info) return Pokedex.db.profile.fDismountForGathering end,
							set = function(info, value) Pokedex.db.profile.fDismountForGathering = value end,
						},
					},
				},
			},
		}
	end
	
	return Pokedex.options;
end


--=========================================================================--
-- Addon Management functions
--=========================================================================--

-- Called when the addon is loaded
function Pokedex:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("PokedexDB", defaults)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Pokedex", GetOptions)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Pokedex", "Pokedex")
	self:RegisterChatCommand("pd", "ChatCommand")
	self:RegisterChatCommand("pokedex", "ChatCommand")
end


function Pokedex:ChatCommand(input)
	if (g_InitState ~= IS_INITIALIZED and self:Initialize("ChatCommand") == IS_UNINITIALIZED) then
		self:Print(L["ERROR: Pokedex failed to initialize correctly. This is usually caused when WoW has invalidated its cache and hasn't finished rebuilding it. Please try this action again later."]);
		return
	end

	if (not input or input:trim() == "") then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(Pokedex, "pd", "Pokedex", input);
	end
end


-- Called when the addon is enabled
function Pokedex:OnEnable()
	-- I'm used to being able to use #DEFINEs and #ifdef debug checks in creating intstrumented
	-- code in a clean way. Prefixing every output with "if (self.db.profile.rgDebugInfo[" is
	-- just going to make me want to claw my eyes out ... so we'll copy out the levels from the
	-- saved profile options into another array with a nice short name.
	for _,v in pairs(self.db.profile.rgDebugInfo) do
		DL[v.category] = v.level;
	end

	-- we should be able to migrate saved data formats immediately, we don't have to wait for init
	self:MigrateSavedData();

	-- check that none of our actions are blocked
	self:RegisterEvent("ADDON_ACTION_BLOCKED");

	-- check that none of our actions are forbidden
	self:RegisterEvent("ADDON_ACTION_FORBIDDEN");

	-- create scanning tooltip for use in scraping tooltip text
	CreateFrame( "GameTooltip", "PokedexScanningTooltip" ); -- Tooltip name cannot be nil
	PokedexScanningTooltip:SetOwner( WorldFrame, "ANCHOR_NONE" );
	PokedexScanningTooltip:AddFontStrings(
		PokedexScanningTooltip:CreateFontString( "$parentTextLeft1",  nil, "GameTooltipText" ),
		PokedexScanningTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" ) );

	-- init data tables and register events and hooks
	self:Initialize("OnEnable");
end


-- Called when the addon is disabled
function Pokedex:OnDisable()
	self:UnregisterEvent("ADDON_ACTION_BLOCKED");
	self:UnregisterEvent("ADDON_ACTION_FORBIDDEN");

	if (g_InitState == IS_INITIALIZED) then
		self:UnregisterEvent("COMPANION_LEARNED");

		self:UnregisterEvent("NEW_TITLE_EARNED");

		self:UnregisterEvent("SKILL_LINES_CHANGED");

		self:UnregisterEvent("CVAR_UPDATE");
		
		self:UnregisterEvent("PLAYER_REGEN_DISABLED");
		self:UnregisterEvent("PLAYER_REGEN_ENABLED");
		self:UnregisterEvent("PLAYER_TARGET_CHANGED");
		
		Pokedex:UnhookAll();
	end
end


-- Initialize all the pet data in the addon
function Pokedex:Initialize(strCaller)
	strCaller = (strCaller or "unknown caller");
	
	if (g_InitState == IS_INITIALIZED) then 
		if (DL[DC_MISC] >= DL_EXCEP) then self:Print("attempt to reinitialize by " .. strCaller); end
		return IS_INITIALIZED;
	end

	if (g_InitState == IS_INITIALIZING) then 
		if (DL[DC_MISC] >= DL_EXCEP) then self:Print("attempt to initialize during initialization by " .. strCaller); end
		return IS_INITIALIZING;
	end

	if (DL[DC_MISC] >= DL_EXCEP) then self:Print("initialization triggered by " .. strCaller); end

	-- if we can't get names for skill, mounts, companions or titles, then 
	-- we know we're probably in the bad cache state and will have to try
	-- initializing again later
	if ( not self:FUpdateSkills() or
	     not self:FUpdateMountInfo() or
	     not self:FUpdateCompanionInfo() or
	     not self:FUpdateTitleInfo() ) then
	     return IS_UNINITIALIZED;
	end
	
	self:UpdateDismountSettings();

	self.db.profile.fFirstBoot = false;

	g_InitState = IS_INITIALIZING;

	-- update mount data if riding skill changes
	self:RegisterEvent("SKILL_LINES_CHANGED");

	-- update data when new mount or companion added	
	self:RegisterEvent("COMPANION_LEARNED");

	-- keep track of current mount or companion
	self:SecureHook(   "CallCompanion",    "CallCompanionHook");
	self:SecureHook("DismissCompanion", "DismissCompanionHook");

	-- update data when new title added	
	self:RegisterEvent("NEW_TITLE_EARNED");

	-- keep track of current title
	self:SecureHook("SetCurrentTitle", "SetCurrentTitleHook");

	-- update dismount settings if the CVAR gets changed
	self:RegisterEvent("CVAR_UPDATE");
	
	-- allow flying dismount if in combat or if you've targeted something attackable
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");

	-- hook tooltip to see if we're about to try to gather
	self:SecureHookScript(GameTooltip, "OnShow", "MainTooltipShow");
	self:SecureHookScript(GameTooltip, "OnHide", "MainTooltipHide");

	self:EchoCounts();
	g_InitState = IS_INITIALIZED;
end

function Pokedex:SafeCall(fn, ...)
	if (g_InitState ~= IS_INITIALIZED and self:Initialize("SafeCall") == IS_UNINITIALIZED) then
		self:Print(L["ERROR: Pokedex failed to initialize correctly. This is usually caused when WoW has invalidated its cache and hasn't finished rebuilding it. Please try this action again later."]);
	else
		return fn(self, ...);
	end
end

--=========================================================================--
--=========================================================================--
--
-- DEBUG AND INFRASTRUCTURE FUNCTIONS
--
--=========================================================================--
--=========================================================================--

function Pokedex:EchoTest()
	local strTest = "test string"
	--self:Print(strTest)
	
	if (DL[DC_DISMOUNT] >= DL_AV) then 
		self:Printf("fManageAutoDismount=%s  fDismountForGathering=%s  fDismountForCombat=%s  fDismountForAttack=%s", 
			tostring(self.db.profile.fManageAutoDismount), tostring(Pokedex.db.profile.fDismountForGathering), 
			tostring(self.db.profile.fDismountForCombat), tostring(self.db.profile.fDismountForAttack));
	end
end
		
function Pokedex:EchoCounts()
	if (DL[DC_MISC] >= DL_BASIC) then 
		self:Printf("Current counts are %i mounts, %i companions and %i titles", #self.db.profile.rgMountInfo, #self.db.profile.rgCompanionInfo, #self.db.profile.rgTitleInfo);
	end
end

function Pokedex:EchoSpeed()
	local nSpeed = GetUnitSpeed("player");
	local nPercent = nSpeed - 7;
	local strOut = "";

	if (nSpeed == 0) then
		strOut = format("%i yards per second", nSpeed);
	elseif (nPercent == 0) then
		strOut = format("%i yards per second, standard run speed", nSpeed);
	elseif (nPercent > 0) then
		nPercent = ((nSpeed - 7) * 100) / 7;
		strOut = format("%.2f yards per second, speed boosted by %.2f%%", nSpeed, nPercent);
	else
		nPercent = (nSpeed * 100) / 7;
		strOut = format("%.2f yards per second, speed reduced to %.2f%% of run speed", nSpeed, nPercent);
	end
	
	strOut = gsub(strOut, "%.00", "");
	self:Print(strOut);
end

function Pokedex:EchoZone()
	SetMapToCurrentZone();

	local strMount, strFly;
	if (IsOutdoors()) then strMount = "can"; else strMount = "cannot"; end
	if (IsFlyableArea()) then strFly = "can"; else strFly = "cannot"; end

	self:Printf("%i  %s  %s. %s mount, %s fly.", GetCurrentMapContinent(), GetZoneText(), GetSubZoneText(), strMount, strFly);
end

function Pokedex:ADDON_ACTION_BLOCKED(_, culprit, action, ...)
	if (culprit ~= "Pokedex") then return end
	self:Printf("ERROR: ADDON_ACTION_BLOCKED calling %s  %s", action, self:StrFromVarArg(...));
end

function Pokedex:ADDON_ACTION_FORBIDDEN(_, culprit, action, ...)
	if (culprit ~= "Pokedex") then return end
	self:Printf("ERROR: ADDON_ACTION_BLOCKED calling %s  %s", action, self:StrFromVarArg(...));
end


function Pokedex:SetDebug(info, value)
	local strName, strValue, index = self:GetArgs(value, 2);
	
	if (strName == nil) then
		self:DebugValues();
		return;
	end
	
	if (index ~= 1e9) then
		self:Print("ERROR: too many parameters passed");
		self:DebugUsage();
		return;
	end
	
	local iValue = tonumber(strValue)
	if (iValue == nil) then
		self:Print("ERROR: no value given for debug level");
		self:DebugUsage();
		return;
	elseif (iValue < DL_NONE or iValue > DL_MAX) then
		self:Printf("ERROR: level_value must be number between %i and %i", DL_NONE, DL_MAX);
		self:DebugUsage();
		return;
	end

	strName = strupper(strName);
	if (strName == "ALL") then
		for k in pairs(self.db.profile.rgDebugInfo) do
			self.db.profile.rgDebugInfo[k].level = iValue;
			DL[self.db.profile.rgDebugInfo[k].category] = iValue;
		end
	elseif (self.db.profile.rgDebugInfo[strName] ~= nil) then
			self.db.profile.rgDebugInfo[strName].level = iValue;
			DL[self.db.profile.rgDebugInfo[strName].category] = iValue;
	else
		self:Print("ERROR: level_name not recognized. Must match an existing value or ALL.");
		self:DebugUsage();
		return;
	end
	
	self:DebugValues();
end

function Pokedex:DebugValues()
	self:Print("current debug levels:");
	for k,v in pairs(self.db.profile.rgDebugInfo) do
		self:Printf("value: %i  name: %s", v.level, k);
		-- self:Printf("lvalue:%i pvalue:%i name:%s", DL[v.category], v.level, k);
	end
end

function Pokedex:DebugUsage()
	self:Print("to see current debug levels:");
	self:Print("  /pd debug ");
	self:Print("to set a level:");
	self:Print("  /pd debug level_name level_value");
end


function Pokedex:StrFromVarArg(...)
	if (select("#", ...) == 0) then return ""; end
	
	local strOut = tostring(select(1, ...));
	for i=2, select("#", ...), 1 do
		strOut = format("%s %s", strOut, tostring(select(i, ...)));
	end

	return strOut;	
end


--=========================================================================--
--=========================================================================--
--
-- SKILL FUNCTIONS
--
--=========================================================================--
--=========================================================================--

function Pokedex:SKILL_LINES_CHANGED()
	local iOldRidingSkill = g_iRidingSkill
	self:FUpdateSkills()

	if (g_iRidingSkill > iOldRidingSkill) then
		self:FUpdateMountInfo()
	end

	-- if you've dropped a gathering skill, we'd want to turn off the related dismount setting
	if ((g_iMiningSkill + g_iHerbalismSkill + g_iSkinningSkill) == 0) then self.db.profile.fDismountForGathering = false; end
end

function Pokedex:FUpdateSkills()
	local iMaxSkills = GetNumSkillLines();
	for iSkill = 1, iMaxSkills, 1 do
		local skillName, _, _, skillRank = GetSkillLineInfo(iSkill);
		if (skillName == nil) then
			if (DL[DC_MISC] >= DL_BASIC) then self:Print("unable to initialize, missing skill names"); end
			return false;
		else
			    if (skillName == L["Riding"])    then g_iRidingSkill    = skillRank;
			elseif (skillName == L["Mining"])    then g_iMiningSkill    = skillRank;
			elseif (skillName == L["Herbalism"]) then g_iHerbalismSkill = skillRank;
			elseif (skillName == L["Skinning"])  then g_iSkinningSkill  = skillRank; end
		end
	end

	if (DL[DC_DISMOUNT] >= DL_AV) then self:Printf("SKILLS  Riding:%i  Mining:%i  Herbalism:%i  Skinning:%i", g_iRidingSkill, g_iMiningSkill, g_iHerbalismSkill, g_iSkinningSkill); end
	return true;
end


--=========================================================================--
--=========================================================================--
--
-- MOUNT AND COMPANION SHARED FUNCTIONS
--
--=========================================================================--
--=========================================================================--

function Pokedex:SuperToggle() 
	if IsOutdoors() then
		self:ToggleMount()
	else
		self:ToggleCompanion()
	end
end

function Pokedex:SummonVendor()
	if (IsOutdoors()) then 
		if (g_iTravelersMammoth ~= 0) then
			if (g_iTravelersMammoth ~= self:GetCurrentMount()) then self:SummonPet("MOUNT", g_iTravelersMammoth); end
		elseif (g_iArgentSquire ~= 0) then
			if (g_iArgentSquire ~= self:GetCurrentCompanion()) then self:SummonPet("CRITTER", g_iArgentSquire); end
		else
			self:Print(L["ERROR: You have no mounts or pets with vendor capabilities"]);
		end
	else
		if (g_iArgentSquire ~= 0) then
			if (g_iArgentSquire ~= self:GetCurrentCompanion()) then self:SummonPet("CRITTER", g_iArgentSquire); end
		elseif (g_iTravelersMammoth ~= 0) then
			self:Print(L["ERROR: You cannot summon your mammoth in this area"]);
		else
			self:Print(L["ERROR: You have no mounts or pets with vendor capabilities"]);
		end
	end
end

function Pokedex:COMPANION_LEARNED(...)
	if (DL[DC_MOUNTS] >= DL_BASIC or DL[DC_PETS] >= DL_BASIC) then self:Print("COMPANION_LEARNED  " .. self:StrFromVarArg(...)); end
	
	self:FUpdateMountInfo()
	self:FUpdateCompanionInfo()
end

function Pokedex:CallCompanionHook(type, index)
	if (type == "CRITTER" ) then
		self:SetCurrentCompanion(index)
	elseif (type == "MOUNT") then
		self:SetCurrentMount(index)
	else
		self:Print(L["ERROR: type error"])
	end
end


function Pokedex:DismissCompanionHook(type)
	if (type == "CRITTER" ) then
		self:SetCurrentCompanion(0)
	elseif (type == "MOUNT") then
		self:SetCurrentMount(0)
	else
		self:Print(L["ERROR: type error"])
	end
end


function Pokedex:SummonPet(strType, iPet)
	if (strType == "CRITTER" ) then
		CallCompanion(strType, iPet)
		self:AnnounceSummon(self.db.profile.rgCompanionInfo[iPet].name)
	elseif (strType == "MOUNT") then
		CallCompanion(strType, iPet)
		--self:AnnounceSummon(self.db.profile.rgMountInfo[iPet].name)

		-- change title on load
		if (self.db.profile.fChangeTitleOnMount) then self:ChangeTitle() end
	else
		self:Print(L["ERROR: type error"])
	end
end


function Pokedex:AnnounceSummon(strName)
	if (not self.db.profile.fAnnounce) then return end

	local iChannel = self.db.profile.iChannel
	
	-- if not in raid, try sending it to party
	if (3 == iChannel and 0 == GetNumRaidMembers()) then
		iChannel = 2
	end
	-- if not in a party, just send it to yourself
	if (2 == iChannel and 0 == GetNumPartyMembers()) then
		iChannel = 1
	end
	
	local strOut
	if (iChannel == 4) then
		strOut = format(L["lets %s know that they have been chosen."], strName);
	else
		strOut = format(L["%s, I choose you!"], strName);
	end
	
	if (iChannel == 1) then
		self:Print(strOut);
	else
		SendChatMessage(strOut, gc_rgstrChannels[iChannel]);
	end
	
end


--=========================================================================--
--=========================================================================--
--
-- MOUNT FUNCTIONS
--
--=========================================================================--
--=========================================================================--

function Pokedex:ToggleMount()
	if (self:GetCurrentMount() > 0 or UnitInVehicle("player")) then
		self:DismissMount()
	else
		self:SummonMount()
	end
end


function Pokedex:DismissMount()
	if UnitInVehicle("player") then
		VehicleExit()
	else
		-- TODO find if this goes through dismiss companion hook for tracking
		Dismount()
	end
end


function Pokedex:SummonMount(fNext, fOther)
	local strCurCast = UnitCastingInfo("player")
	if (strCurCast ~= nil) then
		--if (DL[DC_MOUNTS] >= DL_EXCEP) then self:Print("already casting %s; summon will fail for this reason so ignore this summons attempt"); end
		if (DL[DC_MOUNTS] >= 0) then self:Printf("currently casting %s; summon will fail for this reason so ignore this summons attempt", strCurCast); end
		return
	end

	iCurMount = self:GetCurrentMount();

	local rgiFiltered = {};
	local cTotalRanks = 0;
	local fSkippedCur = false;
	local fSkippedHot = false;
	
	-- We may need to search across multiple lists. For example, if the only 
	-- mount in the set was given a rank of 0. Another case is supporting a 
	-- "style over substance" type feature where 310% and 280% mounts are 
	-- treated as the same category.
	local rgMSFiltered = {};
	local fFlyable = false;
	local fScarab = false;

	-- check if we can fly - cover 99% of situations
	fFlyable = IsFlyableArea() and not IsSwimming();
	
	-- check locations for special rules about what kinds of mounts we can actually summon
	SetMapToCurrentZone();
	continent = GetCurrentMapContinent();
	
	if (1 == continent) then fScarab = strfind(L["Ahn'Qiraj"], GetZoneText()); end
	if (4 == continent and fFlyable) then
		-- if they haven't bought cold weather flying, then they can't fly in northrend
		if (not IsUsableSpell(L["Cold Weather Flying"])) then
			fFlyable = false;
		-- if Wintergrasp is being contested, you can't fly there
		else
			if (GetZoneText() == L["Wintergrasp"]) then
				fFlyable = (nil ~= GetWintergraspWaitTime());
			end			
		end
	end

	local fStyle = false;

	-- flip the type if the user has asked for that
	if fOther then fFlyable = not fFlyable; end

	if (fFlyable) then 
		if (g_rgMountsByType[MS_Flyer310] ~= nil) then rgMSFiltered[#rgMSFiltered+1] = MS_Flyer310 end
		if (g_rgMountsByType[MS_Flyer280] ~= nil) then rgMSFiltered[#rgMSFiltered+1] = MS_Flyer280 end
		if (g_rgMountsByType[MS_Flyer150] ~= nil) then rgMSFiltered[#rgMSFiltered+1] = MS_Flyer150 end
	end

	if (fScarab and g_rgMountsByType[MS_RunnerQiraji] ~= nil) then 
		rgMSFiltered[#rgMSFiltered+1] = MS_RunnerQiraji
	end

	if (g_rgMountsByType[MS_Runner100] ~= nil) then rgMSFiltered[#rgMSFiltered+1] = MS_Runner100 end
	if (g_rgMountsByType[MS_Runner60]  ~= nil) then rgMSFiltered[#rgMSFiltered+1] = MS_Runner60  end
	if (g_rgMountsByType[MS_Runner0]   ~= nil) then rgMSFiltered[#rgMSFiltered+1] = MS_Runner0   end

	if (DL[DC_MOUNTS] >= DL_AV) then
		self:Print("mount types elligible for this zone:")
		for i=1, #rgMSFiltered, 1 do
			self:Print("  " .. gc_rgstrMountSpeeds[rgMSFiltered[i]])
		end
	end
	
	--=====================================================================--
	-- FilterMounts -- Local function that builds list of mounts elligible
	-- for summoning as well as providing a sum of the ranks of those mounts.
	-- Best done as seperate function to support different ways of calling,
	-- next mount versus random mount, and as local function we can reference 
	-- and modify variables in the scope of the calling function.
	-- NOTE: appends to rgiFiltered, so multiple calls can build bigger list
	--=====================================================================--
	local function FilterMounts(msType, iStart, iEnd, fFirstOnly)
		if (iStart == 0 or iStart > iEnd) then
			if (DL[DC_MOUNTS] >= DL_EXCEP) then self:Print("invalid search range"); end
			return;
		end 
		
		if (DL[DC_MOUNTS] >= DL_EXCEP) then self:Printf("building list of eligible %s ...", gc_rgstrMountSpeedDescs[msType]); end
		for iMountOfType = iStart, iEnd, 1 do
			-- translate from index in speed group to all mounts index
			iMount = g_rgMountsByType[msType].rgIndices[iMountOfType];
			
			-- if its ranked as 0, then it gets skipped
			if (self.db.profile.rgMountInfo[iMount].rank == 0) then
				if (DL[DC_MOUNTS] >= DL_EXCEP) then self:Printf("mount %i %s skipped for having rank of 0", iMount, self.db.profile.rgMountInfo[iMount].name); end
			else
				-- if its the current mount, then it is skipped
				if (iMount == iCurMount) then
					if (DL[DC_MOUNTS] >= DL_EXCEP) then self:Printf("mount %i %s skipped for being current mount", iMount, self.db.profile.rgMountInfo[iMount].name); end
					fSkippedCur = true;
				else
					if (fFirstOnly) then
						-- if looking for first good value, then this is it
						rgiFiltered[#rgiFiltered + 1] = iMount;
						cTotalRanks = cTotalRanks + self.db.profile.rgMountInfo[iMount].rank;
						if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("mount %i %s is Next eligible mount", iMount, self.db.profile.rgMountInfo[iMount].name); end
						return;
					-- if hot mount and hotness is enabled then don't add to list, it will be handled seperately
					elseif (self.db.profile.fEnableHotness and iMount == g_iHotMount) then
						if (DL[DC_MOUNTS] >= DL_EXCEP) then self:Printf("mount %i %s skipped for being hot mount", iMount, self.db.profile.rgMountInfo[iMount].name); end
						fSkippedHot = true;
					-- else its put into the pool of summonables
					else
						rgiFiltered[#rgiFiltered + 1] = iMount;
						cTotalRanks = cTotalRanks + self.db.profile.rgMountInfo[iMount].rank;
						if (DL[DC_MOUNTS] >= DL_AV) then self:Printf("mount %i %s added to list of summonable mounts with a rank of %i. Total rank count is %i", iMount, self.db.profile.rgMountInfo[iMount].name, self.db.profile.rgMountInfo[iMount].rank, cTotalRanks); end
					end
				end
			end
		end
	
		if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("list built: %i %s with a total rank of %i", #rgiFiltered, gc_rgstrMountSpeedDescs[msType], cTotalRanks); end
	end
	--=====================================================================--
	-- end of local function FilterMounts
	-- resumption of parent function SummonMount
	--=====================================================================--

	-- Generate filtered list of elligible mounts and total ranks.
	for msFilteredType = 1, #rgMSFiltered, 1 do
		local msType = rgMSFiltered[msFilteredType];
		
		if (fNext and msType == g_rgMountMap[iCurMount].speed) then
			-- We can optimize our search in this case by starting immediately after current
			FilterMounts(msType, g_rgMountMap[iCurMount].index+1, #g_rgMountsByType[msType].rgIndices, true);
			
			-- if none were found in that portion of the list, then start new search from the front of the list
			if (cTotalRanks == 0) then FilterMounts(msType, 1, g_rgMountMap[iCurMount].index, true); end
		else
			FilterMounts(msType, 1, #g_rgMountsByType[msType].rgIndices, fNext);
		end
		
		if (#rgiFiltered ~= 0 or fSkippedHot or fSkippedCur) then 
			break;
		end
	end

	-- if we skipped the hot mount while building a list, nows the time for its heat check
	if (fSkippedHot) then
		local iHeatCheck = math.random(1,100);
		if (self.db.profile.iHotMountPref >= iHeatCheck) then
			if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("mount %i %s passed heat check (rolled %i, needed %i or less) and will be summoned", g_iHotMount, self.db.profile.rgMountInfo[g_iHotMount].name, iHeatCheck, self.db.profile.iHotMountPref); end
			self:SummonPet("MOUNT", g_iHotMount);
			return;
		end
		
		if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("mount %i %s skipped for failing heat check (rolled %i, needed %i or less) as hot mount", g_iHotMount, self.db.profile.rgMountInfo[g_iHotMount].name, iHeatCheck, self.db.profile.iHotMountPref); end
	end

	-- selection returned 0 mounts to choose from	
	if (#rgiFiltered == 0 or cTotalRanks == 0) then 
		-- both values should be in sync
		if (#rgiFiltered ~= cTotalRanks) then self:Print(L["ERROR: only one of #rgiFiltered and cTotalRanks was zero"]); end

		if (fSkippedHot) then
			if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("hot mount %i %s failed heat check but is apparently only one summonable", g_iHotMount, self.db.profile.rgMountInfo[g_iHotMount].name); end
			self:SummonPet("MOUNT", g_iHotMount);
			return
		end
		
		if (fSkippedCur) then
			if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("current mount %i %s is apparently only one summonable; doing nothing", iCurMount, self.db.profile.rgMountInfo[iCurMount].name); end
			return; -- only summonable mount is already summoned
		end
		
		self:Print(L["ERROR: You have no summonable mounts."]);
		return;
	end

	
	-- only one mount to choose from
	if (#rgiFiltered == 1) then
		if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("mount %i %s is apparently only one summonable", rgiFiltered[1], self.db.profile.rgMountInfo[rgiFiltered[1]].name); end
		self:SummonPet("MOUNT", rgiFiltered[1]);
		return;
	end		

	-- multiple mounts
	local cRank = math.random(1,cTotalRanks);
	if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("random roll from 1 to %i produced %i", cTotalRanks, cRank); end

	for _, iNew in ipairs(rgiFiltered) do
		cRank = cRank - self.db.profile.rgMountInfo[iNew].rank;
		if (DL[DC_MOUNTS] >= DL_AV) then self:Printf("mount %i %s's rank of %i brings total down to %i", iNew, self.db.profile.rgMountInfo[iNew].name, self.db.profile.rgMountInfo[iNew].rank, cRank); end
		if (cRank <= 0) then -- found our slot
			self:SummonPet("MOUNT", iNew);
			return;
		end  
	end
	
	if (cRanks > 0) then self:Print(L["ERROR: selection error"]); end
end


-- Hooking CallCompanion and DismissCompanion ensures we'll usually know what 
-- the current state is, but if the user dismisses the mount by clicking off 
-- the buff or casting a spell, we may not get the update, so always get/set 
-- current pet through these functions.
-- TODO: find if there's a way in lua I could scope this variable better.
-- TODO: persist last companion across sessions so that Next starts where you left off
local gr_iLastMountSeen = 0;

function Pokedex:SetCurrentMount(iMount)
	if (DL[DC_MOUNTS] >= DL_BASIC) then self:Printf("Current mount set to: %i %s", iMount, self.db.profile.rgMountInfo[iMount].name); end
	gr_iLastMountSeen = iMount;
	self:SetMountForRanking(gr_iLastMountSeen);
end

function Pokedex:GetCurrentMount()
	if (gr_iLastMountSeen ~= 0) then
		local _, _, _, _, fActive = GetCompanionInfo("MOUNT", gr_iLastMountSeen);
		if (not fActive) then 
			gr_iLastMountSeen = 0; -- might not have an active mount 
		end 
	end
	
	if (gr_iLastMountSeen == 0) then
		for index = 1, GetNumCompanions("MOUNT"), 1 do
			local _, _, _, _, fActive = GetCompanionInfo("MOUNT", index);
			if (fActive) then 
				gr_iLastMountSeen = index;
				break;
			end
		end
	end

	self:SetMountForRanking(gr_iLastMountSeen);
	return gr_iLastMountSeen;
end


function Pokedex:SetMountForRanking(iMount) 
	if (iMount ~= 0) then
		g_iMountType   = g_rgMountMap[iMount].speed;
		g_iMountOfType = g_rgMountMap[iMount].index;
	elseif (#self.db.profile.rgMountInfo == 0) then
		g_iMountType   = 0;
		g_iMountOfType = 0;
	else -- pick first mount of first non-empty category
		for i = 1, MS_MAX, 1 do
			if (g_rgMountsByType[i] ~= nil and #g_rgMountsByType[i]) then
				g_iMountType   = i;
				g_iMountOfType = 1;
				break;
			end
		end
	end
end


function Pokedex:FUpdateMountInfo()
	g_rgstrMountTypes = {};
	g_iMountType = 0;
	g_rgMountsByType = {};
	g_iMountOfType = 0;
	g_rgstrHotMountNames =  { [0] = L["no hot mount"] };
	g_iHotMount = 0;
	g_rgMountMap = { [0] = { speed = 0, index = 0 } };
	
	-- assumption is that order doesn't change except for insertions of new mounts
	-- so we can speed up searching saved data for rankings by starting the search 
	-- at a predicted index
	local iMatch = 1;
	local iNewMount = 0;
	local iActive = 0;
	local rgNew = { [0] = { name = "NONE", id = 0, rank = gc_iDefaultRank } };

	for index = 1, GetNumCompanions("MOUNT"), 1 do
		local idMount, strMountName, idSpell, _, fActive = GetCompanionInfo("MOUNT", index);
		if (strMountName == nil) then
			if (g_InitState == IS_UNINITIALIZED) then
				if (DL[DC_MISC] >= DL_BASIC) then self:Print("unable to initialize, missing mount names"); end
			else
				self:Print(L["ERROR: mount name not available"]);
			end
			return false;
		end
		
		local fMatchMade, iRank = self:GetMountRankFromSavedData(iMatch, idMount);
		local iSpeed, iSpeedAlt = self:GetMountType(strMountName, idSpell);
		
		if (fMatchMade) then 
			iMatch = iMatch + 1;
		else
			if (not self.db.profile.fFirstBoot) then
				self:Printf(L["New mount added: %s"], strMountName);
				iNewMount = index;
				if (self.db.profile.fEnableHotness and self.db.profile.fNewHotness) then
					self.db.profile.idHotMount = idMount;
				end
			end
		end

		if (fActive) then iActive = index; end
		if (idMount == self.db.profile.idHotMount) then g_iHotMount = index; end
		if (strMountName == L["Traveler's Tundra Mammoth"]) then 
		    g_iTravelersMammoth = index ;
		end

		if (DL[DC_MOUNTS] >= DL_AV) then self:Printf("%s  rank: %i  speeds: %s  %s", strMountName, iRank, gc_rgstrMountSpeeds[iSpeed], gc_rgstrMountSpeeds[iSpeedAlt]); end
		
		if (iSpeed ~= MS_Unknown) then
			if (g_rgMountsByType[iSpeed] == nil) then g_rgMountsByType[iSpeed] = { rgIndices = {}, rgNames = {} }; end
			local iNew = 1 + #g_rgMountsByType[iSpeed].rgIndices;

			g_rgMountsByType[iSpeed].rgIndices[iNew] = index;
			g_rgMountsByType[iSpeed].rgNames[iNew] = strMountName;
			
			g_rgMountMap[index] = { ["speed"] = iSpeed, ["index"] = iNew };
		end
		
		if (iSpeedAlt ~= MS_Unknown) then
			if (g_rgMountsByType[iSpeedAlt] == nil) then g_rgMountsByType[iSpeedAlt] = { rgIndices = {}, rgNames = {} }; end
			local iNew = 1 + #g_rgMountsByType[iSpeedAlt].rgIndices;

			g_rgMountsByType[iSpeedAlt].rgIndices[iNew] = index;
			g_rgMountsByType[iSpeedAlt].rgNames[iNew] = strMountName;
		end

		g_rgstrHotMountNames[index] = strMountName;
		rgNew[index] = { name = strMountName, id = idMount, rank = iRank };
	end
 
 	-- now that we've read everything in, blow away old saved ranking with new list
	self.db.profile.rgMountInfo = rgNew;

	-- if they have no mounts, then we need to make some text available for the drop downs
	if (#self.db.profile.rgMountInfo == 0) then
		g_rgstrMountTypes = { [0] = L["no mounts available"] }
		g_rgMountsByType = { [0] = { rgIndices = { [0] = 0 }, rgNames = { [0] = L["no mounts available"] } } }
	-- else, create list of mount types based on those found
	else
		for ms = MS_MAX, 1, -1 do
			if (g_rgMountsByType[ms] ~= nil) then
				g_rgstrMountTypes[ms] = gc_rgstrMountSpeedDescs[ms];
				g_iMountType = ms;
			end
		end
	end

	-- iActive will be either 0 from when we inited, or was set when we found 
	-- an active mount. Calling SetCurrentCompanion ensures our cached value 
	-- for last active mount is correct, as well as correctly tweaking the 
	-- selected drop down items for the no active mount case.
	-- A brand new mount, though, will trump the active mount for selection.
	self:SetCurrentMount(iActive);
	if (iNewMount ~= 0) then self:SetMountForRanking(iNewMount); end
	
	return true;
end


-- return values - fMatchMade, iRank
function Pokedex:GetMountRankFromSavedData(iStart, iID)
	local cLimit = #self.db.profile.rgMountInfo;
	
	if (iStart <= cLimit) then
		for index = iStart, cLimit, 1 do
			if (self.db.profile.rgMountInfo[index].id == iID) then
				return true, self.db.profile.rgMountInfo[index].rank;
			end
		end
		
		for index = 1, iStart - 1, 1 do
			if (self.db.profile.rgMountInfo[index].id == iID) then
				return true, self.db.profile.rgMountInfo[index].rank;
			end
		end
	end
	
	return false, gc_iDefaultRank;
end


function Pokedex:GetMountType(strMountName, idSpell)
	if (nil == strMountName or 0 == idSpell) then 
		self:Print("ERROR - GetMountType: bad params");
		return MS_Unknown, MS_Unknown; 
	end
	
	-- these four erroneously claim only very fast when they are in fact extremely fast
	if (strMountName == L["Ashes of Al'ar"] or
		strMountName == L["Black Proto-Drake"] or
		strMountName == L["Plagued Proto-Drake"] or
		strMountName == L["Violet Proto-Drake"]) then
		return MS_Flyer310, MS_Unknown;

	elseif (strMountName == L["Headless Horseman's Mount"]) then
		if (g_iRidingSkill >= 300) then
			return MS_Flyer280, MS_Runner100;
		elseif (g_iRidingSkill >= 225) then 
			return MS_Flyer150, MS_Runner100;
		elseif (g_iRidingSkill >= 150) then 
			return MS_Runner100, MS_Unknown;
		elseif (g_iRidingSkill >= 750) then 
			return MS_Runner60, MS_Unknown;
		else
			return MS_Runner0, MS_Unknown;
		end

	elseif (strMountName == L["Black Qiraji Battle Tank"]) then
		return MS_Runner100, MS_RunnerQiraji;
	elseif (strfind(strMountName, L["Qiraji"])) then
		return MS_RunnerQiraji, MS_Unknown;
	
	elseif (strfind(strMountName, L["Sea Turtle"])) then
		return MS_Runner0, MS_Unknown; -- TODO: add support for swimming as condition and selection type (like Qiraji scarabs)
	elseif (strfind(strMountName, L["Riding Turtle"])) then
		return MS_Runner0, MS_Unknown;
		
	end
	
	local fExtremelyFast = false;
	local fVeryFast = false;
	local fRidingSkill = false;
	local fFlyer = false;
	
	PokedexScanningTooltip:ClearLines();
	PokedexScanningTooltip:SetHyperlink("spell:" .. idSpell);
	
	for iLines=1,PokedexScanningTooltip:NumLines() do
		local TipTextObject = getglobal("PokedexScanningTooltipTextLeft" .. iLines);
		local strText = TipTextObject:GetText();
		if (DL[DC_MOUNTS] >= DL_AV) then self:Print(strText); end
		
		if (strfind(strText, L["extremely fast"])) then 
			fExtremelyFast = true;
		elseif (strfind(strText, L["very fast"])) then 
			fVeryFast = true;
		elseif (strfind(strText, L["Riding skill"])) then 
			fRidingSkill = true;
		end
		
		if (strfind(strText, L["only"]) and
		    strfind(strText, L["Outland"]) and
		    strfind(strText, L["Northrend"])) then 
			fFlyer = true;
		end
	end

	if (fFlyer) then
		if (fExtremelyFast) then 
			return MS_Flyer310, MS_Unknown;
		elseif (fVeryFast) then 
			return MS_Flyer280, MS_Unknown;
		elseif (fRidingSkill and g_iRidingSkill >= 300) then
			return MS_Flyer280, MS_Unknown;
		else 
			return MS_Flyer150, MS_Unknown; 
		end
	else
		if (fVeryFast) then 
			return MS_Runner100, MS_Unknown;
		elseif (fRidingSkill and g_iRidingSkill >= 150) then 
			return MS_Runner100, MS_Unknown;
		else 
			return MS_Runner60, MS_Unknown;
		end
	end

	self:Print("ERROR - GetMountType: unable to determine type");
	return MS_Unknown, MS_Unknown; 
end


--=========================================================================--
--=========================================================================--
--
-- COMPANION FUNCTIONS
--
--=========================================================================--
--=========================================================================--

function Pokedex:ToggleCompanion()
	if (self:GetCurrentCompanion() == 0) then
		self:SummonCompanion();
	else
		DismissCompanion("CRITTER");
	end
end

function Pokedex:SummonCompanion(fNext)
	local iCurPet = self:GetCurrentCompanion();
	
	if (iCurPet ~= 0) then
		local _, duration = GetCompanionCooldown("CRITTER", iCurPet)
		if (duration ~= 0) then
			if (DL[DC_PETS] >= DL_EXCEP) then self:Print("in cooldown, ignore this summons attempt"); end
			return
		end
	end

	local fHasSnowballs = (GetItemCount(gc_itemIDSnowball) > 0);
	local rgiFiltered = {};
	local cTotalRanks = 0;
	local fSkippedCur = false;
	local fSkippedHot = false;
	
	--=====================================================================--
	-- FilterCompanions -- Local function that builds list of pets elligible
	-- for summoning as well as providing a sum of the ranks of those pets.
	-- Best done as seperate function to support different ways of calling,
	-- next pet versus random pet, and as local function we can reference 
	-- and modify variables in the scope of the calling function.
	-- NOTE: appends to rgiFiltered, so multiple calls can build bigger list
	--=====================================================================--
	local function FilterCompanions(iStart, iEnd, fFirstOnly)
		if (iStart == 0 or iStart > iEnd) then
			if (DL[DC_PETS] >= DL_EXCEP) then self:Print("invalid search range"); end
			return;
		end 
		
		if (DL[DC_PETS] >= DL_EXCEP) then self:Print("building list of eligible pets ..."); end
		for iCompanion = iStart, iEnd, 1 do
			-- if its ranked as 0, then it gets skipped
			if (self.db.profile.rgCompanionInfo[iCompanion].rank == 0) then
				if (DL[DC_PETS] >= DL_EXCEP) then self:Printf("companion %i %s skipped for having rank of 0", iCompanion, self.db.profile.rgCompanionInfo[iCompanion].name); end
			else
				-- if we don't have snowballs and this pet requires one, then it is skipped
				if (not fHasSnowballs and self.db.profile.rgCompanionInfo[iCompanion].snowball) then
					if (DL[DC_PETS] >= DL_EXCEP) then self:Printf("companion %i %s skipped for lack of snowballs", iCompanion, self.db.profile.rgCompanionInfo[iCompanion].name); end
				else
					-- if its the current companion, then it is skipped
					if (iCompanion == iCurPet) then
						if (DL[DC_PETS] >= DL_EXCEP) then self:Printf("companion %i %s skipped for being current companion", iCompanion, self.db.profile.rgCompanionInfo[iCompanion].name); end
						fSkippedCur = true;
					else
						if (fFirstOnly) then
							-- if looking for first good value, then this is it
							rgiFiltered[#rgiFiltered + 1] = iCompanion;
							cTotalRanks = cTotalRanks + self.db.profile.rgCompanionInfo[iCompanion].rank;
							if (DL[DC_PETS] >= DL_BASIC) then self:Printf("companion %i %s is Next eligible pet", iCompanion, self.db.profile.rgCompanionInfo[iCompanion].name); end
							return;
						-- if hot pet and hotness is enabled then don't add to list, it will be handled seperately
						elseif (self.db.profile.fEnableHotness and iCompanion == g_iHotCompanion) then
							if (DL[DC_PETS] >= DL_EXCEP) then self:Printf("companion %i %s skipped for being hot companion", iCompanion, self.db.profile.rgCompanionInfo[iCompanion].name); end
							fSkippedHot = true;
						-- else its put into the pool of summonables
						else
							rgiFiltered[#rgiFiltered + 1] = iCompanion;
							cTotalRanks = cTotalRanks + self.db.profile.rgCompanionInfo[iCompanion].rank;
							if (DL[DC_PETS] >= DL_AV) then self:Printf("companion %i %s added to list of summonable pets with a rank of %i. Total rank count is %i", iCompanion, self.db.profile.rgCompanionInfo[iCompanion].name, self.db.profile.rgCompanionInfo[iCompanion].rank, cTotalRanks); end
						end
					end
				end
			end
		end
	
		if (DL[DC_PETS] >= DL_BASIC) then self:Printf("list built: %i pets with a total rank of %i", #rgiFiltered, cTotalRanks); end
	end
	--=====================================================================--
	-- end of local function FilterCompanions
	-- resumption of parent function SummonCompanion
	--=====================================================================--
	
	-- generate filtered list of elligible pets and total ranks for list
	if (fNext and iCurPet ~= 0) then
		-- We can optimize our search in this case by starting immediately after current
		FilterCompanions(iCurPet+1, #self.db.profile.rgCompanionInfo, true);
		
		-- if none were found in that portion of the list, then start new search from the front of the list
		if (cTotalRanks == 0) then FilterCompanions(1, iCurPet, true); end
	else
		FilterCompanions(1, #self.db.profile.rgCompanionInfo, fNext);
	end


	-- if we skipped the hot pet while building a list, nows the time for its heat check
	if (fSkippedHot) then
		local iHeatCheck = math.random(1,100);
		if (self.db.profile.iHotCompanionPref >= iHeatCheck) then
			if (DL[DC_PETS] >= DL_BASIC) then self:Printf("companion %i %s passed heat check (rolled %i, needed %i or less) and will be summoned", g_iHotCompanion, self.db.profile.rgCompanionInfo[g_iHotCompanion].name, iHeatCheck, self.db.profile.iHotCompanionPref); end
			self:SummonPet("CRITTER", g_iHotCompanion);
			return;
		end
		
		if (DL[DC_PETS] >= DL_BASIC) then self:Printf("companion %i %s skipped for failing heat check (rolled %i, needed %i or less) as hot pet", g_iHotCompanion, self.db.profile.rgCompanionInfo[g_iHotCompanion].name, iHeatCheck, self.db.profile.iHotCompanionPref); end
	end

	-- selection returned 0 pets to choose from	
	if (#rgiFiltered == 0 or cTotalRanks == 0) then 
		-- both values should be in sync
		if (#rgiFiltered ~= cTotalRanks) then self:Print(L["ERROR: only one of #rgiFiltered and cTotalRanks was zero"]); end

		if (fSkippedHot) then
			if (DL[DC_PETS] >= DL_BASIC) then self:Printf("hot companion %i %s failed heat check but is apparently only one summonable", g_iHotCompanion, self.db.profile.rgCompanionInfo[g_iHotCompanion].name); end
			self:SummonPet("CRITTER", g_iHotCompanion);
			return
		end
		
		if (fSkippedCur) then
			if (DL[DC_PETS] >= DL_BASIC) then self:Printf("current companion %i %s is apparently only one summonable; doing nothing", iCurPet, self.db.profile.rgCompanionInfo[iCurPet].name); end
			return; -- only summonable pet is already summoned
		end
		
		self:Print(L["ERROR: You have no summonable companions."]);
		return;
	end

	
	-- only one pet to choose from
	if (#rgiFiltered == 1) then
		if (DL[DC_PETS] >= DL_BASIC) then self:Printf("companion %i %s is apparently only one summonable", rgiFiltered[1], self.db.profile.rgCompanionInfo[rgiFiltered[1]].name); end
		self:SummonPet("CRITTER", rgiFiltered[1]);
		return;
	end		

	-- multiple pets
	local cRank = math.random(1,cTotalRanks);
	if (DL[DC_PETS] >= DL_BASIC) then self:Printf("random roll from 1 to %i produced %i", cTotalRanks, cRank); end

	for _, iNew in ipairs(rgiFiltered) do
		cRank = cRank - self.db.profile.rgCompanionInfo[iNew].rank;
		if (DL[DC_PETS] >= DL_AV) then self:Printf("companion %i %s's rank of %i brings total down to %i", iNew, self.db.profile.rgCompanionInfo[iNew].name, self.db.profile.rgCompanionInfo[iNew].rank, cRank); end
		if (cRank <= 0) then -- found our slot
			self:SummonPet("CRITTER", iNew);
			return;
		end  
	end
	
	if (cRanks > 0) then self:Print(L["ERROR: selection error"]); end
end


-- Hooking CallCompanion and DismissCompanion should ensure that we'll usually
-- know what the current state is.
-- TODO: find out if trying to cast spell by ID can summon dismiss pets bypassing these checks
-- TODO: find if there's a way in lua I could scope this variable better.
-- TODO: persist last companion across sessions so that Next starts where you left off
local gr_iLastCompanionSeen = 0;

function Pokedex:SetCurrentCompanion(iCompanion)
	if (DL[DC_PETS] >= DL_BASIC) then self:Printf("Current companion set to: %i %s", iCompanion, self.db.profile.rgCompanionInfo[iCompanion].name); end
	gr_iLastCompanionSeen = iCompanion;
	self:SetCompanionForRanking(gr_iLastCompanionSeen);
end

function Pokedex:GetCurrentCompanion()
	if (gr_iLastCompanionSeen ~= 0) then
		local _, _, _, _, fActive = GetCompanionInfo("CRITTER", gr_iLastCompanionSeen);
		if (not fActive) then 
			gr_iLastCompanionSeen = 0; -- might not have an active companion 
		end 
	end
	
	if (gr_iLastCompanionSeen == 0) then
		for index = 1, GetNumCompanions("CRITTER"), 1 do
			local _, _, _, _, fActive = GetCompanionInfo("CRITTER", index);
			if (fActive) then 
				gr_iLastCompanionSeen = index;
				break;
			end
		end
	end

	self:SetCompanionForRanking(gr_iLastCompanionSeen);
	return gr_iLastCompanionSeen;
end


function Pokedex:SetCompanionForRanking(iCompanion) 
	if (iCompanion ~= 0) then
		g_iCompanionForRanking = iCompanion;
	elseif (#g_rgstrCompanionNames ~= 0) then
		g_iCompanionForRanking = 1;
	else
		g_iCompanionForRanking = 0;
	end
end


function Pokedex:FUpdateCompanionInfo()
	g_rgstrCompanionNames = {};
	g_iCompanionForRanking = 0;
	g_rgstrHotCompanionNames = { [0] = L["no hot companion"] };
	g_iHotCompanion = 0;
	
	-- assumption is that order doesn't change except for insertions of new pets
	-- so we can speed up searching saved data for rankings by starting the search 
	-- at a predicted index
	local iMatch = 1;
	local iNewPet = 0;
	local rgNew = { [0] = { name = "NONE", id = 0, rank = gc_iDefaultRank, snowball = false } };

	for index = 1, GetNumCompanions("CRITTER"), 1 do
		local idPet, strPetName, idSpell, _, fActive = GetCompanionInfo("CRITTER", index);
		if (strPetName == nil) then
			if (g_InitState == IS_UNINITIALIZED) then
				if (DL[DC_MISC] >= DL_BASIC) then self:Print("unable to initialize, missing companion names"); end
			else
				self:Print(L["ERROR: companion name not available"]);
			end
			return false;
		end

		local fMatchMade, iRank = self:GetCompanionRankFromSavedData(iMatch, idPet);
		local fSnowball = self:GetCompanionAttributes(strPetName, idSpell);

		if (fMatchMade) then 
			iMatch = iMatch + 1;
		elseif (not self.db.profile.fFirstBoot) then
			self:Printf(L["New companion added: %s"], strPetName);
			iNewPet = index;
			if (self.db.profile.fEnableHotness and self.db.profile.fNewHotness) then
				self.db.profile.idHotCompanion = idPet;
			end
		end

		if (fActive) then g_iCompanionForRanking = index; end
		if (idPet == self.db.profile.idHotCompanion) then g_iHotCompanion = index; end
		if (strPetName == L["Argent Squire"] or strPetName == L["Argent Gruntling"]) then 
		    g_iArgentSquire = index;
		end

		g_rgstrCompanionNames[index] = strPetName;
		g_rgstrHotCompanionNames[index] = strPetName;
		rgNew[index] = { name = strPetName, id = idPet, rank = iRank, snowball = fSnowball };
	end

	-- now that we've read everything in, blow away old saved ranking with new list
	self.db.profile.rgCompanionInfo = rgNew;

	-- if they have no pets, then we need to make some text available for the drop down
	if (#g_rgstrCompanionNames == 0) then
		g_rgstrCompanionNames = { [0] = L["no companions available"] };
	end

	-- g_iCompanionForRanking will be either 0 from when we inited, or was set
	-- when we found an active pet. Calling SetCurrentCompanion ensures our 
	-- cached value for last active pet is correct, as well as correctly 
	-- tweaking the selected drop down item for the no active pet case.
	-- A brand new pet, though, will trump the active pet for selection.
	self:SetCurrentCompanion(g_iCompanionForRanking);
	if (iNewPet ~= 0) then self:SetCompanionForRanking(iNewPet); end
	
	return true;
end


-- return values - fMatchMade, iRank
function Pokedex:GetCompanionRankFromSavedData(iStart, iID)
	local cLimit = #self.db.profile.rgCompanionInfo;
	
	if (iStart <= cLimit) then
		for index = iStart, cLimit, 1 do
			if (self.db.profile.rgCompanionInfo[index].id == iID) then
				return true, self.db.profile.rgCompanionInfo[index].rank;
			end
		end
		
		for index = 1, iStart - 1, 1 do
			if (self.db.profile.rgCompanionInfo[index].id == iID) then
				return true, self.db.profile.rgCompanionInfo[index].rank;
			end
		end
	end
	
	return false, gc_iDefaultRank;
end


function Pokedex:GetCompanionAttributes(strPetName, idSpell)
	local fNeedsSnowball = false;
	
	PokedexScanningTooltip:ClearLines();
	PokedexScanningTooltip:SetHyperlink("spell:" .. idSpell);
	
	for iLines=1,PokedexScanningTooltip:NumLines() do
		local TipTextObject = getglobal("PokedexScanningTooltipTextLeft" .. iLines);
		local strText = TipTextObject:GetText();
		-- self:Print(strText)
		
		if strfind(strText, L["Requires a Snowball"]) then 
			fNeedsSnowball = true;
		end
	end
	
	return fNeedsSnowball;
end



--=========================================================================--
--=========================================================================--
--
-- TITLE FUNCTIONS
--
--=========================================================================--
--=========================================================================--

function Pokedex:ChangeTitle()
	local iCurTitle = g_rgTitleMap[self:GetCurrentTitle()];

	local rgiFiltered = {};
	local cTotalRanks = 0;
	local fSkippedCur = false;
	local fSkippedHot = false;

	-- Builds list of elligible titles for change. For mounts and pets, this
	-- is a local function; but we have no plans for NextTitle option or any
	-- other scenarios where we have to make multiple passes at the data.
	if (DL[DC_TITLES] >= DL_EXCEP) then self:Print("building list of eligible titles ..."); end
	for iTitle = 1, #self.db.profile.rgTitleInfo, 1 do
		-- if its ranked as 0, then it gets skipped
		if (self.db.profile.rgTitleInfo[iTitle].rank == 0) then
			if (DL[DC_TITLES] >= DL_EXCEP) then self:Printf("title %i %s skipped for having rank of 0", iTitle, self.db.profile.rgTitleInfo[iTitle].name); end
		else
			-- if its the current title, then it is skipped
			if (iTitle == iCurTitle) then
				if (DL[DC_TITLES] >= DL_EXCEP) then self:Printf("title %i %s skipped for being current title", iTitle, self.db.profile.rgTitleInfo[iTitle].name); end
				fSkippedCur = true;
			else
				-- if hot title and hotness is enabled then don't add to list, it will be handled seperately
				if (self.db.profile.fEnableHotness and iTitle == g_iHotTitle) then
					if (DL[DC_TITLES] >= DL_EXCEP) then self:Printf("title %i %s skipped for being hot title", iTitle, self.db.profile.rgTitleInfo[iTitle].name); end
					fSkippedHot = true;
				-- else its put into the pool of summonables
				else
					rgiFiltered[#rgiFiltered + 1] = iTitle;
					cTotalRanks = cTotalRanks + self.db.profile.rgTitleInfo[iTitle].rank;
					if (DL[DC_TITLES] >= DL_AV) then self:Printf("title %i %s added to list of possible titles with a rank of %i. Total rank count is %i", iTitle, self.db.profile.rgTitleInfo[iTitle].name, self.db.profile.rgTitleInfo[iTitle].rank, cTotalRanks); end
				end
			end
		end
	end

	if (DL[DC_TITLES] >= DL_BASIC) then self:Printf("list built: %i titles with a total rank of %i", #rgiFiltered, cTotalRanks); end
	

	-- if we skipped the hot title while building a list, nows the time for its heat check
	if (fSkippedHot) then
		local iHeatCheck = math.random(1,100);
		if (self.db.profile.iHotTitlePref >= iHeatCheck) then
			if (DL[DC_PETS] >= DL_BASIC) then self:Printf("hot title %i %s passed heat check (rolled %i, needed %i or less) and will be selected", g_iHotTitle, self.db.profile.rgTitleInfo[g_iHotTitle].name, iHeatCheck, self.db.profile.iHotTitlePref); end
			SetCurrentTitle(self.db.profile.idHotTitle);
			return;
		end
		
		if (DL[DC_PETS] >= DL_BASIC) then self:Printf("hot title %i %s failed heat check (rolled %i, needed %i or less) and will not be selected", g_iHotTitle, self.db.profile.rgTitleInfo[g_iHotTitle].name, iHeatCheck, self.db.profile.iHotTitlePref); end
	end

	-- selection returned 0 titles to choose from	
	if (#rgiFiltered == 0 or cTotalRanks == 0) then 
		-- both values should be in sync
		if (#rgiFiltered ~= cTotalRanks) then self:Print(L["ERROR: only one of #rgiFiltered and cTotalRanks was zero"]); end

		if (fSkippedHot) then
			if (DL[DC_PETS] >= DL_BASIC) then self:Printf("hot title %i %s failed heat check but is only eligible title", g_iHotTitle, self.db.profile.rgTitleInfo[g_iHotTitle].name); end
			SetCurrentTitle(self.db.profile.idHotTitle);
			return
		end
		
		if (fSkippedCur) then
			if (DL[DC_PETS] >= DL_BASIC) then self:Printf("current title %i %s is only one eligible; doing nothing", iCurTitle, self.db.profile.rgTitleInfo[iCurTitle].name); end
			return; -- only summonable title is already summoned
		end
		
		self:Print(L["ERROR: You don't have any titles."]);
		return;
	end

	
	-- only one title to choose from
	if (#rgiFiltered == 1) then
		if (DL[DC_PETS] >= DL_BASIC) then self:Printf("title %i %s is apparently only one summonable", rgiFiltered[1], self.db.profile.rgTitleInfo[rgiFiltered[1]].name); end
		SetCurrentTitle(self.db.profile.rgTitleInfo[rgiFiltered[1]].id);
		return;
	end		

	-- multiple titles
	local cRank = math.random(1,cTotalRanks);
	if (DL[DC_PETS] >= DL_BASIC) then self:Printf("random roll from 1 to %i produced %i", cTotalRanks, cRank); end

	for _, iNew in ipairs(rgiFiltered) do
		cRank = cRank - self.db.profile.rgTitleInfo[iNew].rank;
		if (DL[DC_PETS] >= DL_AV) then self:Printf("title %i %s's rank of %i brings total down to %i", iNew, self.db.profile.rgTitleInfo[iNew].name, self.db.profile.rgTitleInfo[iNew].rank, cRank); end
		if (cRank <= 0) then -- found our slot
			SetCurrentTitle(self.db.profile.rgTitleInfo[iNew].id);
			return;
		end  
	end
	
	if (cRanks > 0) then self:Print(L["ERROR: selection error"]); end
end


function Pokedex:SetCurrentTitleHook(idTitle)
	if (idTitle < 1) then
		if (DL[DC_TITLES] >= DL_BASIC) then self:Printf("No title currently showing"); end
	else
		if (DL[DC_TITLES] >= DL_BASIC) then self:Printf("Current title set to: %i %s", idTitle, strtrim(GetTitleName(idTitle))); end
	end
	g_iTitleForRanking = g_rgTitleMap[idTitle];
end

-- return first the title ID and then the sorted order index
function Pokedex:GetCurrentTitle()
	local idTitle = GetCurrentTitle();
	g_iTitleForRanking = g_rgTitleMap[idTitle];

	return idTitle, g_iTitleForRanking;
end


function Pokedex:NEW_TITLE_EARNED()
	self:FUpdateTitleInfo()
end


function Pokedex:FUpdateTitleInfo()
	g_rgstrTitleNames = {};
	g_iTitleForRanking = 0;
	g_rgstrHotTitleNames = { [0] = L["no hot title"] };
	g_iHotTitle = 0;
	g_rgTitleMap = {};

	local rgNew = {};
	local iNewTitle = 0;

	local cTitlesKnown = 0;
	for idTitle = 1, GetNumTitles(), 1 do
		if (1 == IsTitleKnown(idTitle)) then
			cTitlesKnown = cTitlesKnown + 1;

			local strTitleName = GetTitleName(idTitle);
			if (strTitleName == nil) then
				if (g_InitState == IS_UNINITIALIZED) then
					if (DL[DC_MISC] >= DL_BASIC) then self:Print("unable to initialize, missing title names"); end
				else
					self:Print(L["ERROR: title name not available"]);
				end
				return false;
			end

			rgNew[cTitlesKnown] = { name = strtrim(strTitleName), id = idTitle, rank = gc_iDefaultRank };
		end
	end
	
	-- if there are no titles known
	if (#rgNew == 0) then
		-- dummy entry holding default rank in empty case
		rgNew[0] = { name = "NONE", id = 0, rank = gc_iDefaultRank };
		g_rgstrTitleNames[0] = L["no titles available"];
	
	-- otherwise sort the titles, get saved rank values and get index for hot and current titles
	else
		-- sort this table by name to get a nice drop down order
		sort(rgNew, function(a,b) return (a.name < b.name) end);
		if (DL[DC_TITLES] >= DL_AV) then self:Print("Updating Titles ..."); end
		
		-- Now that we're sorted, lets get our saved rank values. Assumption is that 
		-- order doesn't change except for insertions of new titles so we can speed up 
		-- searching saved data for rankings by starting the search at a predicted index
		-- We'll also take advantage of this iteration through to set our drop down lists.
		local iMatch = 1;
		for index = 1, #rgNew, 1 do
			local fMatchMade, iRank = self:GetTitleRankFromSavedData(iMatch, rgNew[index].id);
			
			if (fMatchMade) then
				rgNew[index].rank = iRank;
				iMatch = iMatch + 1;
			elseif (not self.db.profile.fFirstBoot) then
				self:Printf(L["New title added! %s"], rgNew[index].name)
				g_iTitleForRanking = index;
				if (self.db.profile.fEnableHotness and self.db.profile.fNewHotness) then
					self.db.profile.idHotTitle = rgNew[index].id;
				end
			end
			
			if (DL[DC_TITLES] >= DL_AV) then self:Printf("index: %i  name: %s  id: %i  rank %i", index, rgNew[index].name, rgNew[index].id, rgNew[index].rank); end
			
			-- save index of hot pet
			if (self.db.profile.idHotTitle == rgNew[index].id) then 
				g_iHotTitle = index; 
			end
			
			g_rgstrTitleNames[index] = rgNew[index].name;
			g_rgstrHotTitleNames[index] = rgNew[index].name;
		end

		rgNew[0] = { name = L["None"], id = -1, rank = 0 };
		g_rgstrTitleNames[0] = L["None"];
	end
	
	-- dummy entry holding default rank in empty case
	-- rgNew[0] = { name = "NONE", id = 0, rank = gc_iDefaultRank };
	
	-- build a table for reverse lookups, to go from title id to sorted order index
	for key, value in pairs(rgNew) do
		g_rgTitleMap[value.id] = key;
	end

	-- now that the map is set, we can properly init g_iTitleForRanking if
	-- we haven't already pointed it at a newly acquired title
	if (g_iTitleForRanking == 0) then self:GetCurrentTitle(); end

	-- now that we've read everything in, blow away old saved ranking with new list
	self.db.profile.rgTitleInfo = rgNew;
	
	return true;
end


function Pokedex:GetTitleRankFromSavedData(iStart, idTitle)
	local cLimit = #self.db.profile.rgTitleInfo
	local index
	
	if (iStart <= cLimit) then
		for index = iStart, cLimit, 1 do
			if (self.db.profile.rgTitleInfo[index].id == idTitle) then
				return true, self.db.profile.rgTitleInfo[index].rank
			end
		end
		
		for index = 1, iStart - 1, 1 do
			if (self.db.profile.rgTitleInfo[index].id == idTitle) then
				return true, self.db.profile.rgTitleInfo[index].rank
			end
		end
	end
	
	return false, gc_iDefaultRank
end


--=========================================================================--
--=========================================================================--
--
-- AUTO-DISMOUNT FUNCTIONS
--
--=========================================================================--
--=========================================================================--

function Pokedex:CVAR_UPDATE(_, glstr, ...)
	if (DL[DC_DISMOUNT] >= DL_EXCEP) then self:Printf("CVAR_UPDATE EVENT:  %s %s", tostring(glstr), self:StrFromVarArg(...)); end
	if (glstr == L["AUTO_DISMOUNT_FLYING_TEXT"]) then
		self:UpdateDismountSettings();
	end
end


function Pokedex:SetManageAutoDismount(info, value)
	if (g_iAutoDismountFlying == 1 and value) then
		self:Print(L["ALERT - Pokedex has noticed that you have enabled our Safe Dismount feature and has disabled Auto Dismount in Flight accordingly."]);
		g_iAutoDismountFlying = 0;
		SetCVar("autoDismountFlying", 0); --, "AUTO_DISMOUNT_FLYING_TEXT")
	end
	
	Pokedex.db.profile.fManageAutoDismount = value;
end


function Pokedex:UpdateDismountSettings()
	if (GetCVarBool("autoDismountFlying")) then 
		g_iAutoDismountFlying = 1;
	else
		g_iAutoDismountFlying = 0;
	end

	if (g_iAutoDismountFlying == 1 and self.db.profile.fManageAutoDismount) then
		self.Print(L["ALERT - Pokedex has noticed that you have enabled Auto Dismount in Flight and has disabled our Safe Dismount feature accordingly."]);
		self.db.profile.fManageAutoDismount = false;
	end
end


function Pokedex:MainTooltipShow(...)
	-- if we have gathering skills and the feature is turned on
	-- if we're not currently flying then we don't have to worry about monkeying with the setting at all
	if (self.db.profile.fManageAutoDismount and Pokedex.db.profile.fDismountForGathering and IsFlying()) then
		-- if dismount was turned on for a different condition, tooltip scraping will only cause errors
		if ((not self.db.profile.fDismountForCombat or not g_fCanDismountForCombat) and
		    (not self.db.profile.fDismountForAttack or not g_fCanDismountForAttack)) then

			local fGatherable = false;
			for i=1,GameTooltip:NumLines() do
				local mytext = getglobal("GameTooltipTextLeft" .. i);
				local text = mytext:GetText();
				fGatherable = (g_iMiningSkill    > 0 and strfind(text, L["Requires Mining"])) or
							  (g_iHerbalismSkill > 0 and strfind(text, L["Requires Herbalism"])) or
							  (g_iSkinningSkill  > 0 and strfind(text, L["Skinnable"]));
			end

			if (fGatherable and g_iAutoDismountFlying == 0) then
				if (DL[DC_DISMOUNT] >= DL_BASIC) then self:Print("turning on dismount, mousing over gatherable"); end
				g_iAutoDismountFlying = 1;
				SetCVar("autoDismountFlying", 1); --, "AUTO_DISMOUNT_FLYING_TEXT")
			elseif (fGatherable and g_iAutoDismountFlying == 1) then
				if (DL[DC_DISMOUNT] >= DL_EXCEP) then self:Print("WEIRD - new show, but can already dismount"); end
			elseif (not fGatherable and g_iAutoDismountFlying == 1) then
				if (DL[DC_DISMOUNT] >= DL_BASIC) then self:Print("ERROR - can dismount, but tooltip doesn't match"); end
				g_iAutoDismountFlying = 0;
				SetCVar("autoDismountFlying", 0); --, "AUTO_DISMOUNT_FLYING_TEXT")
			end
		end
	end
end


function Pokedex:MainTooltipHide(...)
	-- we only care if gathering feature is turned on
	if (self.db.profile.fManageAutoDismount and Pokedex.db.profile.fDismountForGathering) then
		-- if dismount is already off, we don't have to do anything
		if (g_iAutoDismountFlying == 1) then
			-- if dismount was turned on for a different condition, tooltip scraping will only cause errors
			if ((not self.db.profile.fDismountForCombat or not g_fCanDismountForCombat) and
			    (not self.db.profile.fDismountForAttack or not g_fCanDismountForAttack)) then
				if (DL[DC_DISMOUNT] >= DL_BASIC) then self:Print("dismount turned off, not mousing over gatherable"); end
				g_iAutoDismountFlying = 0;
				SetCVar("autoDismountFlying", 0); --, "AUTO_DISMOUNT_FLYING_TEXT")
			end
		end
	end	
end


function Pokedex:PLAYER_TARGET_CHANGED()
	-- if feature is turned on
	if (self.db.profile.fManageAutoDismount and self.db.profile.fDismountForAttack) then
		-- if we had an attackable target
		if (g_fCanDismountForAttack) then
			-- then only need to change if we lost one
			if (not UnitCanAttack("player", "target") or UnitIsDead("target")) then
				g_fCanDismountForAttack = false;

				-- if in combat, don't clear based on target
				if (not self.db.profile.fDismountForCombat or not g_fCanDismountForCombat) then
					if (DL[DC_DISMOUNT] >= DL_BASIC) then self:Print("dismount turned off, target not attackable"); end
					g_iAutoDismountFlying = 0;
					SetCVar("autoDismountFlying", 0); --, "AUTO_DISMOUNT_FLYING_TEXT")
				end
			end
		-- if not already tracking, then don't start unless we're mounted - hopeful optimization for instances
		elseif (IsMounted()) then
			-- if we have an attackable target
			if (UnitCanAttack("player", "target") and not UnitIsDead("target")) then
				g_fCanDismountForAttack = true;
				
				-- don't need to set if already done for being in combat
				if (not self.db.profile.fDismountForCombat or not g_fCanDismountForCombat) then
					if (DL[DC_DISMOUNT] >= DL_BASIC) then self:Print("turning on dismount, attackable target"); end
					g_iAutoDismountFlying = 1;
					SetCVar("autoDismountFlying", 1); --, "AUTO_DISMOUNT_FLYING_TEXT")
				end
			end
		end
	end
end


function Pokedex:PLAYER_REGEN_DISABLED()
	-- if feature is turned on
	-- if we're not currently mounted then we don't have to worry about monkeying with the setting at all
	if (self.db.profile.fManageAutoDismount and self.db.profile.fDismountForCombat and IsMounted()) then
		g_fCanDismountForCombat = true;

		if (g_iAutoDismountFlying == 0) then
			if (DL[DC_DISMOUNT] >= DL_BASIC) then self:Print("turning on dismount, in combat"); end
			g_iAutoDismountFlying = 1;
			SetCVar("autoDismountFlying", 1); --, "AUTO_DISMOUNT_FLYING_TEXT")
		end
	end
end


function Pokedex:PLAYER_REGEN_ENABLED()
	-- if feature is turned on
	if (self.db.profile.fManageAutoDismount and self.db.profile.fDismountForCombat) then
		-- out of combat, so this will no longer be the reason why we can dismount
		g_fCanDismountForCombat = false;

		-- if can dismount is already 0, then there's nothing left to change
		if (g_iAutoDismountFlying == 1) then
			-- if dismount can occur for attackable target, don't clear for out of combat
			if (not self.db.profile.fDismountForAttack or not g_fCanDismountForAttack) then
				if (DL[DC_DISMOUNT] >= DL_BASIC) then self:Print("dismount turned off, out of combat"); end
				g_iAutoDismountFlying = 0;
				SetCVar("autoDismountFlying", 0); --, "AUTO_DISMOUNT_FLYING_TEXT")
			end
		end
	end	
end
